"""
Fantail
"""

import collections
import inspect


class Fantail(dict):
    def __init__(self, *a, **kw):
        dict.__init__(self)

        for item in a:
            if inspect.isclass(item):
                continue
            elif isinstance(item, dict):
                self.update(item)
            else:
                print item, isinstance(item, Fantail)
                raise Exception("invalid init argument: {}".format(str(item)))
        self.update(kw)

    def __setitem__(self, key, value):
        if isinstance(key, str) and '.' in key:
            keyA, keyB = key.split('.', 1)
            self[keyA][keyB] = value
        elif isinstance(value, dict) and len(value) > 0:
            self[key].update(value)
        else:
            dict.__setitem__(self, key, value)

    def get(self, key, default=None):
        if isinstance(key, str) and '.' in key:
            keyA, keyB = key.split('.', 1)
            if not keyA in self:
                return default
            return self[keyA].get(keyB, default)
        else:
            return dict.get(self, key, default)

    def __getitem__(self, key):
        if isinstance(key, str) and '.' in key:
            keyA, keyB = key.split('.', 1)
            return self[keyA][keyB]

        try:
            return dict.__getitem__(self, key)
        except KeyError:
            return self.__missing__(key)

    def __missing__(self, key):
        self[key] = Fantail()
        return self[key]

    def __reduce__(self):
        return type(self), (Fantail, ), None, None, iter(self.items())

    def copy(self):
        return self.__copy__()

    def __copy__(self):
        return type(self)(Fantail, self)

    def __deepcopy__(self, memo):
        import copy
        return type(self)(Fantail,
                          copy.deepcopy(self.items()))

    def update(self, d=None, **kwargs):
        if d is None:
            pass
        elif isinstance(d, dict):
            for k, v in d.items():
                self[k] = v
        if len(kwargs):
            self.data.update(kwargs)




import os
import logging
import shutil
import tempfile
import unittest
import yaml

import fantail

lg = logging.getLogger(__name__)
lg.setLevel(logging.DEBUG)

test_set_1 = {
    'a': 1,
    'b': 2,
    'c': {'d': 3,
          'e': 4,
          'f': 5},
    'g': [0, 1, 2, 3,
          {'h': 6,
           'i': 7,
           'j': 8,
           }]}

test_set_2 = {
    'a': 18,
    'b': {'k': 9,
          'm': 10
          },
    'g': [0, 1, 2, 3, 4, 5]
}


def d():
    return fantail.Fantail(test_set_1)

class BasicFantailTest(unittest.TestCase):

    def test_load(self):
        fantail.Fantail()

    def test_store_dict(self):
        f = fantail.Fantail()
        f['a'] = 1
        self.assertEqual(f['a'], 1)

test_yaml = """
a:
  b1:
    c1: v1
    c2: v2
    c3: v3
  b2: v4
b: v5
"""

test_dict = yaml.load(test_yaml)

class FantailTest(unittest.TestCase):


    def test_load(self):
        y = fantail.Fantail()

    def test_setgetitem(self):
        y = fantail.Fantail()
        y['a'] = 1
        self.assertEqual(y['a'], 1)

    def test_setlevelgetitem(self):
        y = fantail.Fantail()
        y['a']['b'] = 1
        self.assertEqual(y['a.b'], 1)
        y['c.d'] = 1
        self.assertEqual(y['c']['d'], 1)

    def test_nonstringkeys(self):
        y = fantail.Fantail()
        y[1] = 12
        self.assertEqual(y[1], 12)



    def test_get(self):
        y = fantail.Fantail()
        y['a'] = 1
        self.assertEqual(y.get('a'), 1)
        self.assertEqual(y.get('a', 2), 1)
        self.assertEqual(y.get('b', 3), 3)





class LoaderTest(unittest.TestCase):

    def setUp(self):
        self.tf = tempfile.NamedTemporaryFile(delete=False)
        self.tf.write(test_yaml.encode('utf8'))
        self.tf.close()
        self.test_dir = os.path.join(
            os.path.dirname(__file__), 'data')

    def get_empty_yaco(self):
        return fantail.Fantail()

    def test_dict_loader(self):
        y = fantail.dict_loader(test_dict)
        self.assertEqual(y['a']['b1']['c2'], 'v2')
        self.assertEqual(y['b'], 'v5')

    def test_simple_loader_yaml(self):
        y = fantail.load(test_dict)
        self.assertEqual(y['a']['b1']['c2'], 'v2')
        self.assertEqual(y['b'], 'v5')

    def test_yaml_string_loader(self):
        y = fantail.yaml_string_loader(test_yaml)
        self.assertEqual(y['a']['b1']['c2'], 'v2')
        self.assertEqual(y['b'], 'v5')
        z = fantail.dict_loader(test_dict)
        self.assertEqual(y, z)

    def test_yaml_file_loader(self):
        y = fantail.yaml_file_loader(self.tf.name)
        self.assertEqual(y['a']['b1']['c2'], 'v2')
        self.assertEqual(y['b'], 'v5')
        z = fantail.dict_loader(test_dict)
        self.assertEqual(y, z)

    def test_yaml_file_save(self):
        y = fantail.Fantail()
        y['a']['b']['c'] = 1
        y['d.e'] = '2'

        tf = tempfile.NamedTemporaryFile(delete=False)
        tf.close()
        fantail.yaml_file_save(y, tf.name)
        tf.close()

        z = fantail.yaml_file_loader(tf.name)
        self.assertEqual(y, z)

    def test_dir_loader(self):
        y = fantail.dir_loader(self.test_dir)
        self.assertEqual(y['test']['a'], 1)
        self.assertEqual(y['subdir']['subtest']['e']['g'], 4)
        self.assertEqual(y['subdir']['subsubdir']['subsubtest']['g'], 4)
        self.assertEqual(y['subdir']['subtest']['d'], 'overridden')
        self.assertEqual(y['subdir']['raw'].strip(),
                          'multiline\ntext\nfield')

    def test_package_loader(self):
        y = fantail.package_loader("pkg://fantail/etc")
        #import pprint
        #pprint.pprint(y)
        self.assertEqual(y['Mus'].strip(), 'musculus')
        self.assertEqual(y['sub.Rattus'].strip(), 'swedigicus')
        self.assertEqual(y['sub.Sus'], 'scrofa')
        self.assertEqual(y['sub.test.Gallus'], 'Gallus')

    def tearDown(self):
        os.unlink(self.tf.name)







#!/usr/bin/env python
"""
Fantail provides a configuration object that can be saved to disk as a YAML
file. Fantail can be used to store program configuration and make the
configuration data easily accessible."""

from setuptools.command.test import test as TestCommand
import sys

from distutils.core import setup

extra = {}
if sys.version_info >= (3,):
    extra['use_2to3'] = True

class Tox(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True
    def run_tests(self):
        import tox
        errno = tox.cmdline(self.test_args)
        sys.exit(errno)

DESCRIPTION = "Sliceable key value database wrapper"

setup(name='fantail',
      version='0.0.1',
      description=DESCRIPTION,
      author='Mark Fiers',
      author_email='mark.fiers42@gmail.com',
      url='https://github.com/mfiers/Fantail',
      include_package_data=True,
      packages=['fantail'],
      install_requires = ['PyYAML>=3.0'],
      tests_require = ['tox', 'PyYAML>=3.0'],
      cmdclass = {'test': Tox},
      classifiers = [
          'Development Status :: 5 - Production/Stable',
          'Intended Audience :: Developers',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.2',
          'Programming Language :: Python :: 3.3',
          ],
      **extra
     )
